
/* 
   $Id: rpmmen.h,v 1.10 2001/10/14 08:42:31 pawelk Exp $
 */
/* (C) Pawel Kolodziej (pawelk@pld.org.pl) */
/* Use for your own risk */

#ifndef RPMMEN_H
#define RPMMEN_H

#include <errno.h>
#include <sys/time.h>
#include <rpm/rpmlib.h>
#include <rpm/rpmio.h>
//#include "rl/rpmio_internal.h"
#include <trurl/narray.h>
#include <trurl/nlist.h>
#include "rpmmen_config.h"
#include "../mess.h"

#define TPKGINF_HASHDR		(1 << 0)
#define TPKGINF_BASESYS		(1 << 1)
#define TPKGINF_BASESYS_HEAD	(1 << 2)
/** package can be used for upgrade */
#define TPKGINF_UPGRADE		(1 << 3)
/** package in same version is installed */
#define TPKGINF_EQUAL		(1 << 4)
/** newer packgae is already installed */
#define TPKGINF_OLDER		(1 << 5)
/** package which same version, but other release */
#define TPKGINF_DIFF_RELEASE	(1 << 6)
/** package is on the black list and should not be installed automatically */
#define TPKGINF_BLACK		(1 << 7)

/** wrong name. Packages slected by user "manually" */
#define RPMMEN_FROM_FILE           1
/** slelected from dependencies */
#define RPMMEN_FROM_DEPS           2


#define RPMMEN_CB_CONFLICT_NEW  0
#define RPMMEN_CB_CONFLICT_ADD	1   
#define RPMMEN_CB_CONFLICT_SHOW 2    /* not used by rpmmenlib, but callback
					function should support it */
#define RPMMEN_CB_CONFLICT_UNSATISFIED 3 /* add unsatisfied dependency */

typedef struct {
	char *fname;		/* file name    */
	Header h;		/* file header  */
        char *name;		/* package name */
        char selected;          /*              */
        char flags;		/* TPKGINF_*    */
	char *installedEVR;	/**< EVR of already installed package or null 
				  package is not installed */
	int ps_i;		/* index from PsSet  */
	tn_list * neededBy;     /* packages that need this package */
} TPkgInfo;

struct rpmmen_cb
{
    void (*pkg_open)(const TPkgInfo *);
    void (*pkg_inst_progress)(unsigned long amount, unsigned long total);
    void (*pkg_close)(const TPkgInfo *);
    /* deps(state, pos, max, label) */
    void (*deps_prog)(int, int, int, const char *);
    int  (*conflict)(int action,char *name1,char *ver1,char *rel1,char *name2,
		    char *ver2,int flag2);
    TPkgInfo *(*selectOne)(struct rpmDependencyConflict *conflict);
};

typedef struct {
	tn_array *packages;    /**< elementy to TPkgInfo* */
	rpmdb db;
	char *src;
	char *rootdir;
	char *dbpath;
        struct rpmmen_cb *cb;
	char *local_tocfile;
} TPkgSet;


typedef void (*TStartPackage) (TPkgInfo *);


/* rpmmen: Init rpmmen lib   */
void rpmpkgs_init(TPkgSet * pkgs, char *src, char *dest_, struct rpmmen_cb *cb);

/* rpmmen_close  - close rpm base */
void rpmpkgs_close(TPkgSet *);		/* deinicjacja */

/* rpmmen_opendb - creat directory for base, and open (creat if not exist) rpm
   database files

   RET: 0 - OK
   1 - fail - can't creat directory for base
   2 - fail - can't open (creat) base
 */
int rpmpkgs_opendb(TPkgSet *);

int rpmpkgs_opendb_ro(TPkgSet *);

/* rpmmen_scandir - scan directory, for packages
   RET :  0  - OK
   1  - rpm base open error
   2  - directory read error
 */
int rpmpkgs_scandir(TPkgSet *pkgs);

/* select_from_file : read names of packages to install from file
   ARG  : name - file name
 */
void rpmpkgs_select_from_file(TPkgSet *pkgs, char *name);

/* rpmmen_addPkgsDep - add packages from dependencies
 * RET: count of conflicts
 */
int rpmpkgs_addPkgsDep(TPkgSet *pkgs);

/** return list of lists 
 *  first element of each list is package that causes problem becouse it's
 *  on black list
 **/
tn_list* rpmpkgs_checkBlack(TPkgSet *pkgs);
void rpmmen_freeBlack(tn_list *list);

/* rpmmen_install - install selected packages 
   ARGS : flags - flags for rpmInstallPackage (called from installrpm)
   notify - pointer to function called from rpmInstallPAckage during package
   installation
   startUI - pointer to function called before each package installation
   RET  : 0 - OK (it's only one possible value)
 */
int rpmpkgs_install(TPkgSet *pkgs, int flags);

void rpmpkgs_free(TPkgSet *);

void *rpmpkg_header_entry_get(TPkgSet * pkgs, int num, int tag);
void rpmpkgs_header_entry_free(void *ptr);
void rpmpkgs_removePkgsDep(TPkgSet *pkgs);
unsigned long int rpmpkgs_selected_size(const TPkgSet *pkgs,
                                        int *nselected_pkgs);
void rpmpkgs_selectpackages(TPkgSet *pkgs,char **names,int count);
void rpmpkgs_update_info(TPkgSet *pkgs);
tn_list*  allRequired(TPkgSet *pkgs, char *name,TpkgDescribe *pk);
TpkgDescribe rpmpkgs_getPkgDesc(TPkgSet *pkgs,char *name);
void EVRget(Header h, int release, char *buf);
int rpmpkgs_isInstalled(TPkgSet *pkgs, char *name, char *evr);
#endif
