/* Author : Pawel Kolodziej <pawelk@pld.org.pl> 
 * License: GNU
 */
//#include <pgsql/postgres.h>
//#include <pgsql/libpq-fe.h>
//#include <postgresql/libpq-fe.h>
#include <libpq-fe.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <signal.h>
#include "server_db.h"
#include "net.h"
#include "sql_quote.h"

char *mbtEmail,* dbName,*dbUser;
#define S(a)	sql_quote(a)

/*
 * TODO: improve security level, we don't want ppl to execute
 * arbitrary code on server ... ('%s' -> what if %s="x' SELECT
 * whatever etc... 'x") --mal
 * 
 * It's reather inpossible to exloit this, since query() can execute 
 * only _one_ query; So one can do only smth like:
 * SELECT ..... FROM.... WHERE x=x AND y in select ....... ;
 * It can't destroy anything 
 * But there should be quoting becouse someone can have \' in eq. nickname
 * and w don't want to send illegal query (becouse in that case program will
 * be terminated)  -- pawelk
 * PS I love embedded letters ;) 
 */

static PGconn * conn;

PGresult* doQuery(PGconn *conn,char *query)
{
	PGresult * ret;
	ExecStatusType rc;
printf("\n*zapytanie : $$%s$$\n",query);
	ret=PQexec(conn,query);
	rc=PQresultStatus(ret);
	switch(rc){
		case PGRES_EMPTY_QUERY:
		case PGRES_COPY_OUT:
		case PGRES_COPY_IN:
		case PGRES_BAD_RESPONSE:
		case PGRES_NONFATAL_ERROR:
		case PGRES_FATAL_ERROR:
			printf("ZAPYTANIE #%s#\n",query);
			printf("blad bazy danych: %s\n",
					PQresultErrorMessage(ret));
			PQfinish(conn);
			exit(2);
			break;
		default:
	};
	return ret;
}
		
PGresult* query(PGconn *conn,const char *fmt,...)
{
	char buf[1000];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	return doQuery(conn,buf);
}

int registerUser(char *nick,char *name,char *email,
		int experienceLevel, char *pass)
{
	PGresult * res;
	char insert[1000],select[1000];
	sprintf(select,"SELECT id FROM person WHERE email = '%s'",S(email));
	sprintf(insert,"INSERT INTO person (nick,name,email,password) "
			"VALUES ('%s','%s','%s','%s')",
			S(nick), S(name), S(email), S(pass));
	/* Jesli istenieje koles o takim emailu  to nie
	 * dodawaj nowego */
	if(PQntuples(doQuery(conn,select))){
		printf("TAKI USER JUZ instnieje\n");
		return 0;
	}
			
	doQuery(conn,insert);
	res=doQuery(conn,select);
	if(PQntuples(res)==0){
		printf("BLAD stworzenie user'a\n");
		return 0;
	}
	return (atoi(PQgetvalue(res,0,0)));
}

int registerMachine(char *name, int uid)
{
	PGresult * res;
	char insert[1000],select[1000];
	sprintf(select,"SELECT id FROM machine WHERE name = '%s' AND person_id ='%d'",S(name),uid);
	sprintf(insert,"INSERT INTO machine (name,person_id) "
			"VALUES ('%s','%d')",
			S(name), uid);
			
	doQuery(conn,insert);
	res=doQuery(conn,select);
	if(PQntuples(res)==0){
		printf("BLAD stworzenie maszyny\n");
		return 0;
	}
	return (atoi(PQgetvalue(res,0,0)));
}
/* zwraca numer funkcjonalnosci o danej nazwie i 'approved'>=approved */
int getFeature(char *desc,int approved)
{
	char select[1000];
	int ret;
	PGresult *res;
	sprintf(select,"SELECT id FROM feature WHERE name='%s' and "
			"approved>=%d",S(desc),approved);
	res=doQuery(conn,select);
	if(PQntuples(res)==0)
		return 0;
	ret=atoi(PQgetvalue(res,0,0));
	PQclear(res);
	return ret;
}
		
			
/** Dodaje "funckcjonalnosc" do oglnej listy. zwraca jej numer */
int addNewFeature(int userNo,char *featureDescription)
{
	int ret;
	char insert[1000];

	if((ret=getFeature(featureDescription,0)))
		return ret;
	sprintf(insert,"INSERT INTO feature(name,approved,author) VALUES "
			"('%s',0,%d)",S(featureDescription),userNo);
	doQuery(conn,insert);
	return getFeature(featureDescription,0);
}
	
void addFeatureToPackage(int userNo,int featureNo,int packageNo)
{
	PGresult *res;
	res=query(conn,"SELECT * FROM package_feature WHERE "
			"id_package=%d AND id_feature=%d",
			packageNo,featureNo);
	if(PQntuples(res)==0){
		PQclear(res);
		query(conn,"INSERT INTO package_feature(id_package,"
				"id_feature) VALUES(%d,%d)",
				packageNo,featureNo);
	}
	PQclear(res);
}
		

/* pobiera id pakietu o podanej nazwie. Jesli taki nie figuruje, to go dodaje
 */
int getPackageID(char *name)
{
	PGresult *res;
	int ret;
	res=query(conn,"SELECT id FROM package WHERE name='%s'",S(name));
	if(PQntuples(res)!=0){
		ret=atoi(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
	PQclear(query(conn,"INSERT INTO package (name) VALUES ('%s')",S(name)));
	res=query(conn,"SELECT id FROM package WHERE name='%s'",S(name));
	ret=atoi(PQgetvalue(res,0,0));
	PQclear(res);
	return ret;
}

/* pobiera id architektury o podanej nazwie. Jesli taka nie figuruje, to ja
 * dodaje
 */
int getArchID(char *name)
{
	PGresult *res;
	int ret;
	res=query(conn,"SELECT id FROM arch WHERE name='%s'",S(name));
	if(PQntuples(res)!=0){
		ret=atoi(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
	PQclear(query(conn,"INSERT INTO arch (name) VALUES ('%s')",S(name)));
	res=query(conn,"SELECT id FROM arch WHERE name='%s'",S(name));
	ret=atoi(PQgetvalue(res,0,0));
	PQclear(res);
	return ret;
}


int addReport(int machineNo,TpkgDescribe *package,
		int enviromentCount,TpkgDescribe *enviroment,
		int gradesCount,Tgrade *grades)
{
	PGresult *res;
	int IDpkg,IDarch;
	int IDrep;
	int i;
	IDpkg=getPackageID(package->name);
	IDarch=getArchID(package->arch);
	res=query(conn,"SELECT id FROM REPORT where id_machine=%d AND "
			"id_package=%d AND evr='%s' AND id_arch=%d",
			machineNo,IDpkg,S(package->evr),IDarch);

	
	if(PQntuples(res)!=0){
		res=query(conn,"DELETE FROM report WHERE id = '%s'",
				PQgetvalue(res,0,0));
		PQclear(res);
	}else
		PQclear(res);
	/* - wstaw raport
	 * - wstaw srodowisko
	 * - wstaw opinie
	 */
	
	res=query(conn,"INSERT INTO report (id_machine,id_package,id_arch,evr)"
			" VALUES(%d,%d,%d,'%s')",machineNo,IDpkg,IDarch,
			S(package->evr));
	PQclear(res);
	res=query(conn,"SELECT id FROM REPORT where id_machine=%d AND "
			"id_package=%d AND evr='%s' AND id_arch=%d",
			machineNo,IDpkg,S(package->evr),IDarch);
	IDrep=atoi(PQgetvalue(res,0,0));
	PQclear(res);

		printf("DODAJE SROD 1a\n");
	for(i=0;i<enviromentCount;i++){
		printf("DODAJE SROD 1b\n");
		PQclear(query(conn,"INSERT INTO enviroment(id_report,"
					"id_package,evr,id_arch) VALUES"
					"(%d,%d,'%s',%d)",
					IDrep,
					getPackageID(enviroment[i].name),
					S(enviroment[i].evr),
					getArchID(enviroment[i].arch)));
	}
	for(i=0;i<gradesCount;i++){
		PQclear(query(conn,"INSERT INTO opinion(id_report,id_feature,"
					"mark) VALUES (%d,%d,%d)",
					IDrep,grades[i].featureNo,
					grades[i].grade));
	}
					
return 0;		
}

Tgrade* getReport(char *name, char *evr, char *arch, int *amount, int mid)
{
	PGresult *res;
	Tgrade *ret;
	int pkgId, archId;
	int i;

	pkgId = getPackageID(name);
	archId = getArchID(arch);
	
	res=query(conn,"SELECT id_feature, mark FROM opinion, report WHERE "
			"report.id = opinion.id_report AND "
			"id_machine= '%d' AND id_package = '%d' AND "
			"id_arch = '%d' AND evr ='%s'",mid, pkgId, archId,
			S(evr));
	*amount = PQntuples(res);
	if(*amount == 0){
		PQclear(res);
		return NULL;
	}
	ret = (Tgrade *) malloc(sizeof(Tgrade) * *amount);
	for(i=0;i< *amount; i++){
		ret[i].featureNo = atoi(PQgetvalue(res,i,0));
		ret[i].grade     = atoi(PQgetvalue(res,i,1));
	}
	PQclear(res);
	return ret;
}

/** zwraca zaalokowana tablice intow, zakonczona polem o wartosci '0'
 * Ta tablica okresla numery funkcji przypisanych do danego pakietu */
int* getPackagesFeatures(char *pkgName)
{
	PGresult *res;
	int *ret,retCount,i;
	int IDpkg=getPackageID(pkgName);

	res=query(conn,"SELECT id_feature FROM package_feature WHERE "
			"id_package=%d",IDpkg);
	retCount=PQntuples(res);
	ret=(int *)malloc(sizeof(int)*(retCount+1));
	for(i=0;i<retCount;i++)
		ret[i]=atoi(PQgetvalue(res,i,0));
	ret[retCount]=0;
	PQclear(res);
	return ret;
}
				

char *getPassword(int user_id)
{
	PGresult *res;
	char *ret;
	res=query(conn,"SELECT password FROM person WHERE id=%d",user_id);
	if(PQntuples(res)==0){
		PQclear(res);
		return NULL;	/* no such user? strange... */
	}else{
		ret=strdup(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
}

char *getEmail(int user_id)
{
	PGresult *res;
	char *ret;
	res=query(conn,"SELECT email FROM person WHERE id=%d",user_id);
	if(PQntuples(res)==0){
		PQclear(res);
		return NULL;	/* no such user? strange... */
	}else{
		ret=strdup(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
}

int getUid(char *email)
{
	PGresult *res;
	int ret;
	res=query(conn,"SELECT id FROM person WHERE email='%s'",S(email));
	if(PQntuples(res)==0){
		PQclear(res);
		return -1;	/* no such user? strange... */
	}else{
		ret=atoi(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
}	


char* getFeatureName(int IDfeature)
{
	PGresult *res;
	char *ret;
	res=query(conn,"SELECT name FROM feature WHERE id=%d",IDfeature);
	if(PQntuples(res)==0){
		PQclear(res);
		return NULL;
	}else{
		ret=strdup(PQgetvalue(res,0,0));
		PQclear(res);
		return ret;
	}
	return NULL; /* it never hapens */
	
}

int getMaxFeatureID(int approvedLevel)
{
	PGresult *res;
	
	res=query(conn,"SELECT max(id) FROM feature WHERE approved>=%d",
			approvedLevel);
	if(PQntuples(res)==0)
		return 0;
	return(atoi(PQgetvalue(res,0,0)));
}
			
char ** getPackagesForStatistic(char *pkg,int *count)
{
	PGresult *res;
	int i;
	int pkgID=getPackageID(pkg);
	char **ret;

	res=query(conn,"SELECT DISTINCT package.name FROM package, report, enviroment WHERE report.id_package=%d AND enviroment.id_report=report.id AND package.id=enviroment.id_package",pkgID);
	*count=PQntuples(res);
	ret=malloc(sizeof(char *)* ((*count)? (*count) :1)); /*don't malloc(0)*/
	for(i=0;i<*count;i++)
		ret[i]=strdup(PQgetvalue(res,i,0));
	PQclear(res);
	return ret;
}

void getTable(char ***name, int **id, int *count, char *table)
{	
	PGresult *res;
	int i;
	res = query(conn,"SELECT id, name FROM %s",table);
	*count = PQntuples(res);
	*name = malloc(sizeof(char *) * (*count));
	*id   = malloc(sizeof(int) * (*count));
	for(i=0; i< *count; i++){
		(*name)[i] = strdup(PQgetvalue(res,i,1));
		(*id)[i] = atoi(PQgetvalue(res,i,0));
	}
	PQclear(res);
}

void getMachines(char ***name, int **id, int *count, int uid)
{
	char buf[1000];
	sprintf(buf,"machine WHERE person_id = '%d'",uid);
	getTable(name, id, count, buf);
}

int getMachineId(char *name, int uid)
{
	PGresult *res;
	res=query(conn,"SELECT id FROM machine WHERE person_id='%d' AND "
			"name = '%s'",uid, S(name));
	if(PQntuples(res)==0)
		return 0;
	return(atoi(PQgetvalue(res,0,0)));
}






	

void getDists(char ***name, int **id, int *count)
{
	getTable(name, id,count, "dist");
}	

void getCathegory(char ***name, int **id, int *count)
{
	getTable(name, id,count, "cathegory");
}

char ** getPackagesFromDist(int *count, char ***retEvr, int distId)
{	
	PGresult *res;
	int i;
	char **ret;
	char **evr;	
	
	res=query(conn,"SELECT name, evr FROM package,pkg_evr_dist WHERE "
			"pkg_evr_dist.package_id=package.id AND "
			"dist_id='%d'",distId);
	*count=PQntuples(res);
	ret=malloc(sizeof(char *)* ((*count)? (*count) :1)); /*don't malloc(0)*/
	evr=malloc(sizeof(char *)* ((*count)? (*count) :1)); /*don't malloc(0)*/
	*retEvr=evr;
	for(i=0;i<*count;i++){
		ret[i]=strdup(PQgetvalue(res,i,0));
		evr[i]=strdup(PQgetvalue(res,i,1));
	}
	PQclear(res);
	return ret;

}

#if 0
char ** getPackagesFromTest(int *count, char ***retEvr)
{
	PGresult *res;
	int i;
	char **ret;
	char **evr;	
	
	res=query(conn,"SELECT name, evr FROM package, in_test WHERE in_test.id_package=package.id");
	*count=PQntuples(res);
	ret=malloc(sizeof(char *)* ((*count)? (*count) :1)); /*don't malloc(0)*/
	evr=malloc(sizeof(char *)* ((*count)? (*count) :1)); /*don't malloc(0)*/
	*retEvr=evr;
	for(i=0;i<*count;i++){
		ret[i]=strdup(PQgetvalue(res,i,0));
		evr[i]=strdup(PQgetvalue(res,i,1));
	}
	PQclear(res);
	return ret;
}
#endif


/* not used yet */
/* statystyka pakietu "pkg" ze wzgldu na wlasnosc featureID oraz pakiet
 * srodowiskowy envID 
 * RET: static buffer */
int * getStatistic(TpkgDescribe *pkg, TpkgDescribe *env,int featureID)
{
	PGresult *res;
	static int ret[12];
	PQprintOpt opt={1,1,0,0,0,0,"|","","",NULL};
	int pkgID=getPackageID(pkg->name);
	int pkgArch=getArchID(pkg->arch);
	int envID=getPackageID(env->name);
	int mark;
	int i;
	int map[]={0,-1,-1,1,-1,-1,-1,2,-1,-1,3};
#if 0	
	/* to zapytanie daje nam statystyke ze wzgldu na wszystkie wlasciwosci
	 * na raz */
	res=query(conn,"SELECT COUNT(opinion.id),mark,opinion.id_feature,"
			"       evrcmp(enviroment.evr,'%s')"
			" FROM opinion,report,enviroment "
/*1raport wlasciwego*/	" WHERE report.id_package=%d AND "
/*1pakietu */		"       report.id_arch=%d AND "
/*1---------*/		"       (evrcmp(report.EVR,'%s')=0) AND "
/*2wlasciwa opinia */	"       opinion.id_report=report.id AND "
/*3ten pakiet z   */	"       enviroment.id_report=report.id AND "
/*3srodowiska */	"       enviroment.id_package=%d  "
			" GROUP BY mark,opinion.id_feature,"
			"         evrcmp(enviroment.evr,'%s')",
			env->evr,pkgID,pkgArch,pkg->evr,envID,env->evr);
#endif
	/* a tutaj tylko jedna wlasciwosc na raz ( i jeden pakiet z srodowiska)
	 * Tak wic na jadna komletna statustyke (wszystkie wlasciwosci i
	 * pakiety z srodowiska trzeba bedzie wykonac mnstwo zapyta 
	 * TODO: zrobic to lepiej*/
	
	res=query(conn,"SELECT COUNT(opinion.id),mark,"
			"       evrcmp(enviroment.evr,'%s')"
			" FROM opinion,report,enviroment "
/*1raport wlasciwego*/	" WHERE report.id_package=%d AND "
/*1pakietu */		"       report.id_arch=%d AND "
/*1---------*/		"       (evrcmp(report.EVR,'%s')=0) AND "
/*2wlasciwa opinia */	"       opinion.id_report=report.id AND "
/*3ten pakiet z   */	"       enviroment.id_report=report.id AND "
/*3srodowiska */	"       enviroment.id_package=%d  AND "
			"       opinion.id_feature=%d "
			" GROUP BY mark,opinion.id_feature,"
			"         evrcmp(enviroment.evr,'%s')",
			S(env->evr),pkgID,pkgArch,S(pkg->evr),envID,featureID,
			S(env->evr));
	/* zwraca tablice :
	 * evrcmp         -1              0              1
	 *           /-------------\  /------------\  /-------------\
	 * oceny       0  3  7  10     0  3  7  10      0  3  7 10 
	 *
	 * ilosc takich
	 * opini
	 * (indeks     0  1  2  3      4  5  6   7      8  9  10 11
	 * w tablicy)
	 */ 
	/* pola w res: 0 - count
	 *             1 - mark
	 *             2 - evrcmp
	 */
	PQprint(stdout,res,&opt);

	for(i=0;i<12;i++)
		ret[i]=0;
	for(i=0;i<PQntuples(res);i++){
		mark=atoi(PQgetvalue(res,i,1));
		printf("ocena przed mapowaniem %d\n",mark);
		if(mark==-1)     /* mark: "I don't know" */
			continue;
		if(mark<0 || mark>10){
			MYLOG(("mark out of range\n"));			
			continue;
		}
		/*XXX*/
		mark=map[mark];
		if(mark==-1){
			MYLOG(("invalid mark\n"));
			continue;
		}
		printf("@@@@@@@@@ makr : %d\n",mark);
		ret[mark+(atoi(PQgetvalue(res,i,2))*4)+4]=
			atoi(PQgetvalue(res,i,0));
	}
	return ret;
}

			
void serverOpenDB()
{
	char buf[100];
	sprintf(buf,"dbname=%s",dbName);
	conn=PQconnectdb(buf);
	if(PQstatus(conn)==CONNECTION_BAD){
		printf("Nie mozna podlaczyc sie do bazy (%s)!\n",
				PQerrorMessage(conn));
		PQfinish(conn);
		exit(1);
	}
}

void serverCloseDB()
{
	PQfinish(conn);
}

#if 0
	
int main()
{

	int i;
	TpkgDescribe pk={"wucaaha","0.1","i586"};
	TpkgDescribe en[]={
		{"rpm","4.0","i586"},
		{"glibc","2.0.7","i686"}
	};
	Tgrade gr[]={{1,11},{2,12},{3,13},{5,55}};
	conn=PQconnectdb("dbname=mop");
	if(PQstatus(conn)==CONNECTION_BAD){
		printf("Nie mozna podlaczyc sie do bazy (%s)!\n",
				PQerrorMessage(conn));
		PQfinish(conn);
		exit(1);
	}
	i=registerUser("pawelk__","Koaalodziej","Pawel","pawelk@pld.org.pl",0);
	printf("Nowy user ma id=%d\n",i);
	printf("nowa wlasnosc %d\n",addNewFeature(5,"wsparcie dla aipv6"));
	printf("pakietNo: %d\n",getPackageID("apache"));
	printf("pakietNo: %d\n",getPackageID("mc"));
	
	addFeatureToPackage(1,addNewFeature(7,"ssev"),getPackageID("mc"));

	query(conn,"BEGIN");
	addReport(11,&pk,2,en,4,gr);
	query(conn,"END");
/*	
int addReport(int userNo,TpkgDescribe *package,
		int enviromentCount,TpkgDescribe *enviroment,
		int gradesCount,Tgrade *grades)
*/
	PQfinish(conn);
	return 0;
			
}
	
#endif

struct user *user_head;
int listen_sock;

int capable(struct user *u, int cap)
{
	if (u == 0 || u->access == 0)
		return 0;
	if (strchr(u->access, '0'))
		return 1;
		
	return strchr(u->access, cap) != 0;
}

void kill_user(struct user *u)
{
	struct user *p;

	if (u == user_head)
		user_head = u->next;
	else {
		for (p = user_head; p->next; p = p->next)
			if (p->next == u)
				break;
		p->next = u->next;
	}

	MYLOG(("user 'userno = %d' [%d] killed\n", u->user_no, u->fd));

	yfree(u->access);
	yfree(u->cookie);
	close(u->fd);
	yfree(u);
}

void init_sock()
{
	char pbuf[sizeof("65536\0")];
	struct addrinfo hints, *res, *res0;
	int gerr;

	snprintf(pbuf, sizeof(pbuf), "%d", PORT);
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;
	gerr = getaddrinfo(NULL, pbuf, &hints, &res0);
	if (gerr) {
	    fprintf(stderr, "getaddrinfo(NULL, \"%d\", ...): %s\n",
		    PORT, gai_strerror(gerr));
	    exit(1);
	}

	for (res = res0; res; res = res->ai_next) {
	    listen_sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	    if (listen_sock == -1) {
		if (res->ai_next)
		    continue;
		else {
		    perror("socket");
		    freeaddrinfo(res0);
		    exit(1);
		}
	    }
	    
	    if (bind(listen_sock, res->ai_addr, res->ai_addrlen)) {
		if (res->ai_next) {
		    close(listen_sock);
		    continue;
		} else {
		    perror("bind");
		    freeaddrinfo(res0);
		    exit(1);
		}
	    }
	    break;
	}
	freeaddrinfo(res0);
	listen(listen_sock, 8);
}

void accept_new()
{
	int fd;
	struct user *u;

	/* FIXME: logging : source address */
	fd = accept(listen_sock, 0, 0);
	if (fd == -1) {
		perror("accept");
		exit(1);
	}
	
	u = malloc(sizeof(struct user));
	u->fd = fd;
	u->next = user_head;
	user_head = u;
	u->user_no = 0;
	u->machine_no = 0;
	u->access = 0;
	u->cookie = 0;
	u->why_killed = 0;	/* we don't want user to get killed
				 * until they say sth :) 
				 */

	MYLOG(("new user accepted\n"));
}

void main_loop()
{
	int err, max_fd;
	fd_set rd;
	struct user *p;

	for (;;) {
		FD_ZERO(&rd);
		max_fd = listen_sock;
		FD_SET(listen_sock, &rd);
		for (p = user_head; p; p = p->next) {
			FD_SET(p->fd, &rd);
			if (p->fd > max_fd)
				max_fd = p->fd;
		}
		max_fd++;
		err = select(max_fd, &rd, 0, 0, 0);
		if (err < 0) {
			perror("select");
			exit(2);
		}
		if (FD_ISSET(listen_sock, &rd))
			accept_new();
			
		again:
		for (p = user_head; p; p = p->next) 
			if (FD_ISSET(p->fd, &rd)) {
				FD_CLR(p->fd, &rd);
				handle_user(p);
				/* user list can have been modified */
				goto again;
			}
	}
}

void int_sig(int s)
{
	serverCloseDB();
	MYLOG(("exiting on signal %d\n", s));
	exit(0);
}

int main(int ac, char *av[])
{

	if(ac!=3){
		printf("usage: mop_server mbtemail dbname \n");
		exit(1);
	}
	mbtEmail = strdup(av[1]);
	dbName = strdup(av[2]);
	
	signal(SIGTERM, int_sig);
	signal(SIGINT, int_sig);
	signal(SIGHUP, SIG_IGN);
	signal(SIGPIPE, SIG_IGN);
	
	init_sock();
	serverOpenDB();
	MYLOG(("in main loop\n"));
	main_loop();
	/* NEVER REACHED */
	return 0;
}
